using System; 
using System.Collections;
using System.Collections.Specialized;
using System.IO;
using System.Text;

using NewsComponents.Utils;
using NewsComponents.Feed; 

namespace NewsComponents.News{




	/// <summary>
	/// Class for parsing NNTP results sent by a server 
	/// </summary>
	public class NntpParser{

		/// <summary>
		/// Parses the results of an NNTP LIST request into an ArrayList of newsgroup names.
		/// </summary>
		/// <returns>An arraylist containing a list of newsgroups</returns>
		public static StringCollection GetNewsgroupList(Stream newsgroupListStream){
		
			StringCollection col = new StringCollection();
			string newsgroup;
			int len; 

			TextReader reader    = new StreamReader(newsgroupListStream); 
			//skip NNTP status code
			reader.ReadLine();

			// Read and display lines from the file until the end of 
			// the file is reached.
			while ( ((newsgroup = reader.ReadLine()) != null) && 
				(newsgroup.StartsWith(".")!= true) && (newsgroup != String.Empty)){
				len = newsgroup.IndexOf(" "); 
				len = (len == -1 ? len = newsgroup.Length : len);
				col.Add(newsgroup.Substring(0, len)); 
			}		

			reader.Close(); 
		
			return col; 
		}


		/// <summary>
		/// Reads the list of articles from the stream and returns the feed item.
		/// </summary>	
		/// <param name="f">Information about the feed. This information is updated based
		/// on the results of processing the feed. </param>
		/// <param name="feedStream">A stream containing an RSS feed.</param>				
		/// <param name="cachedStream">Flag states update last retrieved date on feed only 
		/// if the item was not cached. Indicates whether the lastretrieved date is updated
		/// on the feedsFeed object passed in. </param>
		/// <returns>An ArrayList of NewsItem objects</returns>		
		public static ArrayList GetItemsForNewsGroup(feedsFeed f, Stream newsgroupListStream, bool cachedStream) {

			int readItems = 0; 
			ArrayList items = new ArrayList(); 
			NewsItem item; 
			string currentLine, title, author, parentId, headerName, headerValue, id; 
			StringBuilder content = new StringBuilder(); 
			DateTime pubDate; 
			int colonPos; 
			TextReader reader    = new StreamReader(newsgroupListStream); 				


			while(reader.ReadLine()!= null){ //skip NNTP status code
			
				/* headers loop */				
				title = author = parentId = id = null; 
				pubDate = DateTime.Now;
				content.Remove(0, content.Length); 


				while(((currentLine = reader.ReadLine()) != null) && 
						(!currentLine.Trim().Equals(String.Empty))){ //TODO: Get rid of Trim() call								
						
					colonPos = currentLine.IndexOf(":");

					if(colonPos > 0){
						headerName  = currentLine.Substring(0, colonPos).ToLower();
						headerValue = currentLine.Substring(colonPos + 2); //skip whitespace after colon

						switch(headerName){
							case "subject" : 
								title = headerValue; 
								break; 
							case "from": 
								author = headerValue;
								break; 
							case "references":	//some posts may have multiple ids in References header
								int spaceIndex = headerValue.IndexOf(" "); 
								parentId = (spaceIndex == -1 ? headerValue : headerValue.Substring(0, spaceIndex)); 
								break;
							case "date": 
								pubDate = DateTimeExt.Parse(headerValue); 
								break; 
							case "message-id":
								id = headerValue;
								break; 
							default: 
								break;//ignore other NNTP headers
						}

					}//if(colonPos > 0}					

				}//

				/* content loop */
				
				while(((currentLine = reader.ReadLine()) != null) && 
					(currentLine.StartsWith(".")!= true)){					
					content.Append(currentLine); 				
				}

				item = new NewsItem(f, title, content.ToString(), author, id,  parentId); 
				items.Add(item); 

				reader.ReadLine(); //read blank line after the item

			}


			//update last retrieved date on feed only if the item was not cached.
			if (!cachedStream) {
				f.lastretrieved = new DateTime(DateTime.Now.Ticks);
				f.lastretrievedSpecified = true;
			}
			
			//any new items in feed? 
			if((items.Count== 0) || (readItems == items.Count)){
				f.containsNewMessages = false; 
			}else{
				f.containsNewMessages = true; 
			}

			reader.Close(); 

			return items; 
		}
	
	
	
	}


}