#region CVS Version Header
/*
 * $Id: RelationCosmos.cs,v 1.5 2005/01/18 17:36:41 carnage4life Exp $
 * Last modified by $Author: carnage4life $
 * Last modified at $Date: 2005/01/18 17:36:41 $
 * $Revision: 1.5 $
 */
#endregion

using System;
using System.Diagnostics;
using System.Collections;
using System.Threading;

using NewsComponents.Collections;
using NewsComponents.Utils;
using NewsComponents.Threading;

namespace NewsComponents.RelationCosmos
{
	/// <summary>
	/// Summary description for RelationCosmos.
	/// </summary>
	public class RelationCosmos
	{
		#region ctor's
		/// <summary>
		/// Initializer
		/// </summary>
		public RelationCosmos()	{
			worker = new PriorityThread();
			allRelations = new Relations();
			registeredRelations = new Relations();
			allIncomingRelations = new RelationLists();
			registeredIncomingRelations = new RelationLists();
		}
		#endregion

		#region public static methods
		/// <summary>
		/// Provides a empty RelationList instance.
		/// </summary>
		public static RelationList EmptyRelationList { get { return emptyRelationList; } }
		/// <summary>
		/// This provides a empty RelationHRefDictionary instance
		/// </summary>
		public static RelationHRefDictionary EmptyRelationHRefDictionary { get { return emptyHRefDictionary; } }
		/// <summary>
		/// A timespan that express the time subtracted from a known point in time to get
		/// another related relation point in time adjusted. Default is one second.
		/// </summary>
		public static TimeSpan DefaultRelationTimeCorrection { get { return defaultRelationTimeCorrection; } }
		/// <summary>
		/// A default date representing the unknown point in time of a relation.
		/// </summary>
		public static DateTime UnknownPointInTime { get { return DateTime.MinValue; } }
		/// <summary>
		/// A hashtable table for adding URLs. This allows us to implement the equivalent 
		/// of string interning without the performance overhead of String.Intern
		/// </summary>
		public static StringTable UrlTable{ get { return urlTable; } }
		#endregion
		
		#region public methods
		/// <summary>
		/// Add a new Relation to the RelationCosmos. 
		/// The relation(s) should be registered in time order oldest first to
		/// prevent lost relationships.
		/// </summary>
		/// <param name="relation">Relation to add</param>
		/// <exception cref="ArgumentNullException">If relation or relation.HRef is null</exception>
		public void Add(RelationBase relation)  {
			
			if (relation == null) 
				throw new ArgumentNullException("relation");
			
			if (relation.HRef == null)
				return;	// nothing we can refer back

			lock(syncRoot) {
				
				string href = relation.HRef;
				
				try {
					if (registeredRelations.Contains(href)) {
						Remove(relation);
					}

					if (deepCosmos && !allRelations.ContainsKey(href)) {
						allRelations.Add(href, relation);
					}

					// add the real (known) outgoing relations
					if (relation.outgoingRelations.Count > 0) {
						
						foreach (string hrefOut in relation.outgoingRelations.Keys) {
							
							if (hrefOut != null && hrefOut.Length > 0) {
								
								if (deepCosmos && !allRelations.Contains(hrefOut) ) {
									allRelations.Add(hrefOut, new RelationProxy(hrefOut, relation.PointInTime, relation.PointInTimeIsAdjustable));
								} 
								
								if (deepCosmos) {
									if (href != hrefOut && allRelations.Contains(hrefOut)) {
										RelationBase known = allRelations[hrefOut];
										// A relation can only contain refs to OLDER relation entries.
										// So we correct the unknown PointInTime to be the same dateTime of 
										// the known entry subtract by the defaultRelationTimeCorrection.
										if (known.PointInTime > relation.PointInTime){
											AdjustRelationPointInTime(relation, known, adjustPointInTime);
										}
										AddToRelationList(relation.HRef, known, allIncomingRelations);
									}
								}
								
								if (href != hrefOut && registeredRelations.Contains(hrefOut)) {
									RelationBase known = registeredRelations[hrefOut];
									// A relation can only contain refs to OLDER relation entries.
									// So we correct the unknown PointInTime to be the same dateTime of 
									// the known entry subtract by the defaultRelationTimeCorrection.
									if (known.PointInTime > relation.PointInTime) {
										AdjustRelationPointInTime(relation, known, adjustPointInTime);
									}
									AddToRelationList(known.HRef, relation, registeredIncomingRelations);
								}
							}
						}//foreach

					}

					// ensure new relation have a valid point in time entry.
					if (relation.PointInTime == UnknownPointInTime)
						relation.SetInternalPointInTime(DateTime.UtcNow);	// keep the movable information

					// If we get relation(s) to add in unordered manner, that relation can be older then some known.
					// So we have to loop over yet known relations and test, if the known outgoing contains
					// reference(s) to the new one. If so, we add an entry to registeredIncomingRelations.
					foreach (RelationBase known in registeredRelations.Values) {
						
						if (known.outgoingRelations.ContainsKey(href)) {
							if (known.CompareTo(relation) <= 0) {	// known is OLDER
								// A relation can only contain refs to OLDER relation entries.
								// so we correct the unknown PointInTime to be the same dateTime of 
								// the known entry subtract by the defaultRelationTimeCorrection.
								AdjustRelationPointInTime(relation, known, adjustPointInTime);
							} // else known is NEWER
							AddToRelationList(relation.HRef, known, registeredIncomingRelations);
						}//contains(relation)
					}//foreach

					//add to registered collection
					registeredRelations.Add(href, relation);

				} catch (Exception ex) {
					Trace.Write("RelationCosmos.Add() exception:"+ ex.Message);
				}
			}			
		}
		
		/// <summary>
		/// Add a range of <c>RelationBase</c> objects
		/// </summary>
		/// <param name="relations">RelationBase[]</param>
		public void AddRange(RelationBase[] relations)  {
			this.AddRange((IList)relations);
		}
		/// <summary>
		/// Add a range of <c>RelationBase</c> objects
		/// </summary>
		/// <param name="relations">IList</param>
		public void AddRange(IList relations)  {
			RelationBase[] relationsToAdd = new RelationBase[relations.Count];
			relations.CopyTo(relationsToAdd, 0);
			worker.QueueUserWorkItem(new WaitCallback(this.ThreadRunAddRange), relationsToAdd, 0);
		}
		
		/// <summary>
		/// Add a range of <c>RelationBase</c> objects
		/// </summary>
		/// <param name="relations">IList</param>
		protected void InternalAddRange(IList relations)  {
			if (relations == null) 
				throw new ArgumentNullException("relations");
			
			foreach (RelationBase relation in relations) {

				if (StringHelper.EmptyOrNull(relation.HRef))
					continue;	// nothing we can refer back

				string href = relation.HRef;

				lock(syncRoot) {
				
					
					if (registeredRelations.Contains(href)) {
						Remove(relation);
					}

					if (deepCosmos && !allRelations.ContainsKey(href)) {
						allRelations.Add(href, relation);
					}

					// add the real (known) outgoing relations
					if (relation.outgoingRelations.Count > 0) {
							
						foreach (string hrefOut in relation.outgoingRelations.Keys) {
								
							if (hrefOut != null && hrefOut.Length > 0) {
									
								if (deepCosmos && !allRelations.Contains(hrefOut) ) {
									allRelations.Add(hrefOut, new RelationProxy(hrefOut, relation.PointInTime, relation.PointInTimeIsAdjustable));
								} 
									
								if (deepCosmos) {
									if (href != hrefOut && allRelations.Contains(hrefOut)) {
										RelationBase known = allRelations[hrefOut];
										// A relation can only contain refs to OLDER relation entries.
										// So we correct the unknown PointInTime to be the same dateTime of 
										// the known entry subtract by the defaultRelationTimeCorrection.
										if (known.PointInTime > relation.PointInTime){
											AdjustRelationPointInTime(relation, known, adjustPointInTime);
										}
										AddToRelationList(relation.HRef, known, allIncomingRelations);
									}
								}
									
								if (href != hrefOut && registeredRelations.Contains(hrefOut)) {
									RelationBase known = registeredRelations[hrefOut];
									// A relation can only contain refs to OLDER relation entries.
									// So we correct the unknown PointInTime to be the same dateTime of 
									// the known entry subtract by the defaultRelationTimeCorrection.
									if (known.PointInTime > relation.PointInTime) {
										AdjustRelationPointInTime(relation, known, adjustPointInTime);
									}
									AddToRelationList(known.HRef, relation, registeredIncomingRelations);
								}
							}
						}//foreach

						//reduce mem. usage:
						//relation.OutgoingRelations.TrimToSize();

					}

				}//lock
				
				// ensure new relation have a valid point in time entry.
				if (relation.PointInTime == UnknownPointInTime)
					relation.SetInternalPointInTime(DateTime.UtcNow);	// keep the movable information

			}//foreach(RelationBase)	

			// If we get relation(s) to add in unordered manner, that relation can be older then some known.
			// So we have to loop over yet known relations and test, if the known outgoing contains
			// reference(s) to the new one. If so, we add an entry to registeredIncomingRelations.
			
			// in that case we simply add any new relation:
			if (registeredRelations.Count == 0)	{	
			
				lock(syncRoot) { 
					foreach (RelationBase relation in relations) {
						//add to registered collection
						if (!StringHelper.EmptyOrNull(relation.HRef) && !registeredRelations.ContainsKey(relation.HRef))
							registeredRelations.Add(relation.HRef, relation);
					}
				}
			
			} else {

				// The "CopyTo()" construct prevents against InvalidOpExceptions/ArgumentOutOfRange, 
				// if we modify registeredRelations here and from other thread(s)
				RelationBase[] values;
			
				//long secs = 0;
				//ProfilerHelper.StartMeasure(ref secs);

				lock (syncRoot) {
					values = new RelationBase[registeredRelations.Count];
					registeredRelations.Values.CopyTo(values, 0);	// takes 0.01 secs to copy 15,000 items...
				}

				//Trace.WriteLine(ProfilerHelper.StopMeasureString(secs), "RelationCosmos copy of values(" + values.Length.ToString() + ")");

				foreach (RelationBase known in values) {

					foreach (RelationBase relation in relations) {
					
						string href = relation.HRef;
						if (href == null || href.Length == 0)
							continue;

						lock(syncRoot) {
							if (known.outgoingRelations.ContainsKey(href)) {
								if (known.CompareTo(relation) <= 0) {	// known is OLDER
									// A relation can only contain refs to OLDER relation entries.
									// so we correct the unknown PointInTime to be the same dateTime of 
									// the known entry subtract by the defaultRelationTimeCorrection.
									AdjustRelationPointInTime(relation, known, adjustPointInTime);
								}// else known is NEWER
								AddToRelationList(href, known, registeredIncomingRelations);
							}//contains(relation)

							//add to registered collection
							if (!registeredRelations.ContainsKey(href))
								registeredRelations.Add(href, relation);
						}
				
					}//foreach relation
				}//foreach known
			}//if
		}

		/// <summary>
		/// Remove a relation from the RelationCosmos.
		/// </summary>
		/// <param name="relation">Relation to be removed</param>
		public void Remove(RelationBase relation) {

			if (relation == null || relation.HRef == null) {
				return;
			}
			
			lock(syncRoot) {
				try {

					if (allRelations.ContainsKey(relation.HRef)) 
						allRelations.Remove(relation.HRef);
					
					if (registeredRelations.ContainsKey(relation.HRef)) 
						registeredRelations.Remove(relation.HRef);

					// remove from the real (known) outgoing relations
					if (relation.OutgoingRelations.Count > 0) {
						foreach (string hrefOut in relation.outgoingRelations.Keys) {
							if (hrefOut != null && hrefOut.Length > 0) {
								if (allIncomingRelations.ContainsKey(hrefOut)) {
									RemoveFromRelationList(hrefOut, relation, allIncomingRelations);
								}
								if (registeredIncomingRelations.ContainsKey(hrefOut)) {
									RemoveFromRelationList(hrefOut, relation, registeredIncomingRelations);
								}
								if (allRelations.ContainsKey(hrefOut)) 
									allRelations.Remove(hrefOut);
							}
						}
					} else {	
						// no outgoing links
					}

				} catch (Exception ex) {
					Trace.Write("RelationCosmos.Remove() exception:"+ ex.Message);
				}
			}			
		}

		/// <summary>
		/// Overloaded. Remove a amount of RelationBase objects from the RelationCosmos.
		/// </summary>
		/// <param name="relations">To be removed RelationBase object's</param>
		public void RemoveRange(IList relations) { 
			foreach (RelationBase r in relations) {
				Remove(r);
			}
		}
		/// <summary>
		/// Overloaded. Remove a amount of RelationBase objects from the RelationCosmos.
		/// </summary>
		/// <param name="relations">To be removed RelationBase object's</param>
		public void RemoveRange(RelationBase[] relations) {
			foreach (RelationBase r in relations) {
				Remove(r);
			}
		}
		/// <summary>
		/// Clear all internal collections.
		/// </summary>
		public void Clear() { 
			allRelations.Clear();
			registeredRelations.Clear();
			allIncomingRelations.Clear();
			registeredIncomingRelations.Clear();
		}

		/// <summary>
		/// Returns a list of relations, that are known in RelationCosmos and pointing to
		/// the relation provided.
		/// </summary>
		/// <param name="relation">The object implementing RelationBase</param>
		/// <param name="excludeRelations">List of relations, 
		/// that should be excluded in that check</param>
		/// <returns>RelationList</returns>
		public RelationList GetIncoming(RelationBase relation, IList excludeRelations) {
			if (relation == null || relation.HRef == null)
				return RelationCosmos.emptyRelationList;

			lock(syncRoot) {
				try {
					if (excludeRelations == null)
						excludeRelations = (IList) RelationCosmos.emptyRelationList;

					// check incoming:
					RelationList list = registeredIncomingRelations[relation.HRef];
					if (list != null && list.Count > 0) {
						RelationList returnList = new RelationList(list.Count);
						foreach (RelationBase r in list) {
							if (r != relation && !excludeRelations.Contains(r)) {
								returnList.Add(r);
							}
						}
						return returnList;
					}

				} catch (Exception ex) {
					Trace.Write("RelationCosmos.GetIncoming() exception:"+ ex.Message);
				}

				return RelationCosmos.emptyRelationList;
			}			
		}

		/// <summary>
		/// Returns a list of relations, that are known in RelationCosmos and that 
		/// the relation provided points to.
		/// </summary>
		/// <param name="relation">The object implementing RelationBase</param>
		/// <param name="excludeRelations">List of relations, 
		/// that should be excluded in that check</param>
		/// <returns>RelationList</returns>
		public RelationList GetOutgoing(RelationBase relation, IList excludeRelations) { 
			if (relation == null || relation.HRef == null)
				return RelationCosmos.emptyRelationList;

			lock(syncRoot) {
				try {
					if (excludeRelations == null)
						excludeRelations = (IList)RelationCosmos.emptyRelationList;

					// check incoming:
					RelationHRefDictionary list = relation.outgoingRelations;
					if (list != null && list.Count > 0) {
						RelationList returnList = new RelationList(list.Count);
						foreach (string hrefOut in list.Keys) {
							if (hrefOut != relation.HRef && !RelationListContainsHRef(excludeRelations, hrefOut) &&
								registeredRelations.Contains(hrefOut)) {
								returnList.Add(registeredRelations[hrefOut]);
							}
						}
						return returnList;
					}

				} catch (Exception ex) {
					Trace.Write("RelationCosmos.GetOutgoing() exception:"+ ex.Message);
				}

				return RelationCosmos.emptyRelationList;
			}			
		}
		/// <summary>
		/// Returns a list merged of incoming and outging relations.
		/// </summary>
		/// <param name="relation">The object implementing RelationBase</param>
		/// <param name="excludeRelations">List of relations, 
		/// that should be excluded in that check</param>
		/// <returns>RelationList</returns>
		public RelationList GetIncomingAndOutgoing(RelationBase relation, IList excludeRelations) { 
			RelationList returnList = new RelationList(this.GetIncoming(relation, excludeRelations));
			returnList.AddRange(this.GetOutgoing(relation, excludeRelations));
			if (returnList.Count > 0) {
				return returnList;
			}
			return RelationCosmos.emptyRelationList;
		}
		
		/// <summary>
		/// Return true, if the <c>relation</c> has any incoming or outgoing relations 
		/// (registered/added to RelationCosmos). 
		/// </summary>
		/// <param name="relation">Relation to check</param>
		/// <param name="excludeRelations">List of strings with relation.HRef's, 
		/// that should be excluded in that check</param>
		/// <returns>True, if any relation was found, else false</returns>
		public bool HasIncomingOrOutgoing(RelationBase relation, IList excludeRelations) { 
			
			if (relation == null || relation.HRef == null)
				return false;

			lock(syncRoot) {
				try {
					if (excludeRelations == null)
						excludeRelations = RelationCosmos.emptyRelationList;

					// check outgoing:
					foreach (string hrefOut in relation.outgoingRelations.Keys) {
						if (hrefOut != null && hrefOut != relation.HRef && 
							!RelationListContainsHRef(excludeRelations, hrefOut) && registeredRelations.Contains(hrefOut)) {
							return true;
						}
					}

					// check incoming:
					RelationList list = registeredIncomingRelations[relation.HRef];
					if (list != null) {
						foreach (RelationBase r in list) {
							if (r != relation && !excludeRelations.Contains(r)) {
								return true;
							}
						}
					}

				} catch (Exception ex) {
					Trace.Write("RelationCosmos.HasIncomingOrOutgoing() exception:"+ ex.Message);
				}
				return false;
			}			
		}

		/// <summary>
		/// Not yet fully implemented/supported!
		/// </summary>
		public bool DeepCosmos { get { return deepCosmos; } set { deepCosmos = value; } }
		/// <summary>
		/// Set this to true, if RelationCosmos should try to adjust the PointInTime properties
		/// of added Relations, if they are adjustable. It can do so, because of the knowledge
		/// about the other Relations and their relationships to other.
		/// </summary>
		public bool AdjustPointInTime { get { return adjustPointInTime; } set { adjustPointInTime = value; } }
		#endregion

		#region private static methods
		private static RelationList GetRelationList(string key, RelationLists fromRelations) { 
			RelationList list = fromRelations[key];
			if (list == null) { 
				list = new RelationList(3);
				fromRelations.Add(key, list);
			}
			return list;  
		}
		private static void AddToRelationList(string key, RelationBase relation, RelationLists toRelations) { 
			RelationList list = GetRelationList(key, toRelations);
			if (!list.Contains(relation)) { 
				list.Add(relation);
			}
		}
		private static void RemoveFromRelationList(string key, RelationBase relation, RelationLists fromRelations) { 
			RelationList list = fromRelations[key];
			if (list != null)  {
				int removeIndex = list.IndexOf(relation);
				if (removeIndex >= 0) {
					list.RemoveAt(removeIndex);
				}
				if (list.Count == 0) {
					fromRelations.Remove(key);
				}
			}
		}
		private static bool RelationListContainsHRef(IList relationList, string href) {
			foreach (RelationBase r in relationList) {
				if (r.HRef == href)
					return true;
			}
			return false;
		}
		/// <summary>
		/// If the PointInTime of parameter <c>relation</c> is greater than <c>registeredRelation</c>,
		/// we adjust the them. We favor to adjust only relations with <c>PointInTimeIsAdjustable</c> set to true.
		/// But if <c>force</c> is true, we always adjust <c>relation</c> also if both <c>PointInTimeIsAdjustable</c>
		/// are false.</summary>
		/// <param name="relation">Relation to adjust</param>
		/// <param name="registeredRelation">Relation to compare to</param>
		/// <param name="force">Force a point in time adjustment.
		/// <c>PointInTimeIsAdjustable</c> of both <c>relation</c> and <c>registeredRelation</c> are
		/// considered before this happens</param>
		private static void AdjustRelationPointInTime(RelationBase relation, RelationBase registeredRelation, bool force) {
			if (relation.PointInTime > registeredRelation.PointInTime) {
				if (relation.PointInTimeIsAdjustable) {
					relation.SetInternalPointInTime(registeredRelation.PointInTime.Subtract(defaultRelationTimeCorrection));
				} else if (registeredRelation.PointInTimeIsAdjustable && relation.PointInTime != UnknownPointInTime) {
					registeredRelation.SetInternalPointInTime(relation.PointInTime.Add(defaultRelationTimeCorrection));
				} else if (force) {
					relation.SetInternalPointInTime(registeredRelation.PointInTime.Subtract(defaultRelationTimeCorrection));
				}
			}
		}

		#endregion
		
		#region private methods
		private void ThreadRunAddRange(object state) {
			IList relations = (IList)state;
			this.InternalAddRange(relations);
		}
		#endregion

		#region private vars
		//private static readonly log4net.ILog _log = RssBandit.Common.Logging.Log.GetLogger(typeof(RelationCosmos));
		private static RelationList emptyRelationList = RelationList.ReadOnly(new RelationList(0));
		private static TimeSpan defaultRelationTimeCorrection = new TimeSpan(100);	// 100 nanosecs == 1 sec
		private static RelationHRefDictionary emptyHRefDictionary = new RelationHRefDictionary(0);
		private static StringTable urlTable = new StringTable(); 

		private Relations allRelations;
		private Relations registeredRelations;
		private RelationLists allIncomingRelations;
		private RelationLists registeredIncomingRelations;
		private PriorityThread worker;

		private bool deepCosmos = false;
		private bool adjustPointInTime = false;
		private object syncRoot = new Object();
		#endregion
	}

}
