#region CVS Version Header
/*
 * $Id: RssHelper.cs,v 1.9 2005/01/11 10:15:27 t_rendelmann Exp $
 * Last modified by $Author: t_rendelmann $
 * Last modified at $Date: 2005/01/11 10:15:27 $
 * $Revision: 1.9 $
 */
#endregion

using System;
using System.Collections;
using System.IO; 
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Xml; 
using System.Diagnostics;

using NewsComponents.Feed;

namespace NewsComponents.Utils
{


	/// <summary>
	/// Enumeration used to tell which field a NewsItemComparer should use when comparing NewsItem objects
	/// </summary>
	public enum NewsItemSortField{
		Title, 
		Subject, 
		Date,
		CommentCount, 
		Author
	}

	/// <summary>
	/// RssHelper
	/// </summary>
	public sealed class RssHelper {

		/// <summary>
		/// </summary>
		public const string NsSlashModules = "http://purl.org/rss/1.0/modules/slash/";
		
		/// <summary>
		/// For spec. see http://dublincore.org/2003/03/24/dces#
		/// </summary>
		public const string NsDcElements    = "http://purl.org/dc/elements/1.1/";

		/// <summary>
		/// For spec. see http://wellformedweb.org/news/wfw_namespace_elements/
		/// </summary>
		public const string NsCommentAPI = "http://wellformedweb.org/CommentAPI/";

		// object needed to create XmlElement's, etc.
		private static XmlDocument elementCreator = new XmlDocument();

		/// <summary>
		/// Converts a list of NewsItem's to a list of NewsItem.Link's
		/// </summary>
		/// <param name="items"></param>
		/// <returns></returns>
		static public IList ItemListToHRefList(IList items) {
			ArrayList retList = new ArrayList(items.Count);
			foreach (NewsItem item in items) {
				if (!StringHelper.EmptyOrNull(item.Link))
					retList.Add(item.Link);
			}
			return retList;
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="item"></param>
		/// <param name="elementName"></param>
		/// <returns></returns>
		static public string GetDcElementValue(NewsItem item, string elementName) {
			return GetOptionalElementValue(item, elementName, NsDcElements);
		}

		/// <summary>
		/// TODO
		/// </summary>
		/// <param name="item"></param>
		/// <returns></returns>
		static public string GetHashCode(NewsItem item) {
			if (item == null) return null;
			System.Text.StringBuilder newHash = new System.Text.StringBuilder();
			
			if (item.Feed != null && item.Feed.link != null && item.Feed.link.Length > 0)
				newHash.Append(item.Feed.link.GetHashCode().ToString());

			if (item.Link != null && item.Link.Length > 0)
				newHash.Append(item.Link.GetHashCode().ToString());
			
			if (item.Title != null && item.Title.Length > 0)
				newHash.Append(item.Title.GetHashCode().ToString());

			if (item.HasContent)
				newHash.Append(item.Content.GetHashCode().ToString());
			
			return newHash.ToString();
		}

		static private string GetOptionalElementValue(NewsItem item, string elementName, string elementNamespace) {
			if (item.OptionalElements == null || item.OptionalElements.Count == 0)
				return null;

			string retStr = null;
			XmlElement elem = GetOptionalElement(item, elementName, elementNamespace);
			if (elem != null)
				retStr = elem.InnerText; 

			return retStr;
		}

		/// <summary>
		/// Returns the optional XmlElement if found within the NewsItem.OptionalElements list,
		/// else null.
		/// </summary>
		/// <param name="item">NewsItem</param>
		/// <param name="elementName">String</param>
		/// <param name="elementNamespace">String</param>
		/// <returns></returns>
		static public XmlElement GetOptionalElement(NewsItem item, string elementName, string elementNamespace) {
			if (item == null || item.OptionalElements == null)
				return null;
			return GetOptionalElement(item.OptionalElements, elementName, elementNamespace);
		}
		
		/// <summary>
		/// Returns the optional XmlElement if found within the NewsItem.OptionalElements list,
		/// else null.
		/// </summary>
		/// <param name="item">NewsItem</param>
		/// <param name="qName">XmlQualifiedName</param>
		/// <returns>XmlElement</returns>
		static public XmlElement GetOptionalElement(NewsItem item, XmlQualifiedName qName) {
			if (item == null || item.OptionalElements == null || qName == null)
				return null;
			return GetOptionalElement(item.OptionalElements, qName.Name, qName.Namespace);
		}

		/// <summary>
		/// Returns the optional XmlElement if found within the provided list,
		/// else null.
		/// </summary>
		/// <param name="elements"></param>
		/// <param name="elementName"></param>
		/// <param name="elementNamespace"></param>
		/// <returns></returns>
		static public XmlElement GetOptionalElement(IDictionary elements, string elementName, string elementNamespace) {
			if (elements == null || elements.Count == 0)
				return null;
			
			XmlQualifiedName qname = GetOptionalElementKey(elements, elementName, elementNamespace);
			if (qname != null) {
				XmlElement elem  = (XmlElement) elements[qname];
				if(elem != null) {
					return elem; 
				}
			}
			return null;
		}
		/// <summary>
		/// Returns the optional XmlElement if found within the provided list,
		/// else null.
		/// </summary>
		/// <param name="elements"></param>
		/// <param name="qName">XmlQualifiedName</param>
		/// <returns>XmlQualifiedName</returns>
		static public XmlQualifiedName GetOptionalElementKey(IDictionary elements, XmlQualifiedName qName) {
			if (elements == null || elements.Count == 0 || qName == null)
				return null;
			return GetOptionalElementKey(elements, qName.Name, qName.Namespace);
		}
		
		/// <summary>
		/// Returns the optional XmlElement if found within the provided list,
		/// else null.
		/// </summary>
		/// <param name="elements"></param>
		/// <param name="elementName"></param>
		/// <param name="elementNamespace"></param>
		/// <returns>XmlQualifiedName</returns>
		static public XmlQualifiedName GetOptionalElementKey(IDictionary elements, string elementName, string elementNamespace) {
			if (elements == null || elements.Count == 0)
				return null;

			//have to do this because XmlQualifiedName.Equals() is stupid
			foreach(XmlQualifiedName qname in elements.Keys){						
				if (qname.Namespace.Equals(elementNamespace) && qname.Name.IndexOf(elementName) >= 0) {
					return qname;
				}
			}
			return null;
		}

		/// <summary>
		/// Creates and returns a new XmlElement with the specified
		/// prefix, elementName, elementNamespace and value.
		/// </summary>
		/// <param name="prefix"></param>
		/// <param name="elementName">String</param>
		/// <param name="elementNamespace">String</param>
		/// <param name="value">String</param>
		/// <returns>XmlElement</returns>
		static public XmlElement CreateXmlElement(string prefix, string elementName, string elementNamespace, string value) {
			XmlElement e = elementCreator.CreateElement(prefix, elementName, elementNamespace); 
			e.InnerText  = value; 
			return e;
		}

		/// <summary>
		/// Returns the default NewsItem comparer.
		/// </summary>
		/// <returns>Ascending sorting NewsItemComparer (by Date)</returns>
		static public IComparer GetComparer() {
			return new NewsItemComparer();
		}
		/// <summary>
		/// Returns a NewsItem comparer.
		/// </summary>
		/// <param name="sortDescending">Set to False, if it should sort ascending (by date), 
		/// else true</param>
		/// <returns></returns>
		static public IComparer GetComparer(bool sortDescending) {
			return new NewsItemComparer(sortDescending);
		}


		/// <summary>
		/// Returns a NewsItem comparer.
		/// </summary>
		/// <param name="sortDescending">Set to False, if it should sort ascending, 
		/// else true</param>
		/// <param name="sortField">indicates which field on the NewsItem object should be used for 
		/// sorting</param>
		/// <returns></returns>
		static public IComparer GetComparer(bool sortDescending, NewsItemSortField sortField) {
			return new NewsItemComparer(sortDescending, sortField);
		}

		internal class NewsItemComparer: IComparer {
		
			#region private fields

			private bool sortDescending;
			private NewsItemSortField sortField;
			 
			#endregion 

			#region Constructors 

			public NewsItemComparer():this(false, NewsItemSortField.Date)  {}

			public NewsItemComparer(bool sortDescending):this(sortDescending, NewsItemSortField.Date){}
			
			public NewsItemComparer(NewsItemSortField sortField):this(false, sortField) {}

			public NewsItemComparer(bool sortDescending, NewsItemSortField sortField) {
				this.sortField = sortField;
				this.sortDescending = sortDescending;
			}

			#endregion

			#region Implementation of IComparer
			public int Compare(object x, object y) {
				
				NewsItem ri1 = x as NewsItem, ri2 = y as NewsItem;
				
				if (ri1 == null || ri2 == null)
					return 0;
				
				if (this.sortDescending){

					switch(this.sortField){
						case NewsItemSortField.Date:
							return DateTime.Compare(ri2.Date, ri1.Date);

						case NewsItemSortField.Author:
							return String.Compare(ri2.Author, ri1.Author); 

						case NewsItemSortField.CommentCount:
							return  ri2.CommentCount.CompareTo(ri1.CommentCount); 

						case NewsItemSortField.Subject:
							return String.Compare(ri2.Subject, ri1.Subject); 

						case NewsItemSortField.Title:
							return String.Compare(ri2.Title, ri1.Title); 

						default: 
							return 0; 
					}
				}else{
					
					switch(this.sortField){
						case NewsItemSortField.Date:
							return DateTime.Compare(ri1.Date, ri2.Date);

						case NewsItemSortField.Author:
							return String.Compare(ri1.Author, ri2.Author); 

						case NewsItemSortField.CommentCount:
							return ri1.CommentCount.CompareTo(ri2.CommentCount); 

						case NewsItemSortField.Subject:
							return String.Compare(ri1.Subject, ri2.Subject); 

						case NewsItemSortField.Title:
							return String.Compare(ri1.Title, ri2.Title); 

						default: 
							return 0; 
					}
				}
			}
		
			#endregion
		}

	
	}

}
