#region CVS Version Header
/*
 * $Id: IdentityNewsServerManager.cs,v 1.8 2005/02/03 21:07:09 t_rendelmann Exp $
 * Last modified by $Author: t_rendelmann $
 * Last modified at $Date: 2005/02/03 21:07:09 $
 * $Revision: 1.8 $
 */
#endregion

using System;
using System.Collections;
using System.IO;
using System.Xml; 
using System.Xml.Schema;
using System.Xml.Serialization;
using System.Windows.Forms;
using System.Diagnostics;
using System.Net;
using System.Web;

using NewsComponents;
using NewsComponents.Feed;
using NewsComponents.News;
using NewsComponents.Net;
using NewsComponents.Utils;

using RssBandit.WinGui.Forms;

namespace RssBandit
{

	#region IdentityNewsServerManager
	/// <summary>
	/// Summary description for IdentityNewsServerManager.
	/// </summary>
	public class IdentityNewsServerManager
	{
		// logging/tracing:
		private static readonly log4net.ILog _log = RssBandit.Common.Logging.Log.GetLogger(typeof(IdentityNewsServerManager));

		private static UserIdentity anonymous;

		private RssBanditApplication app;
		private string cachePath;

		private IdentityNewsServerManager() {}
		public IdentityNewsServerManager(RssBanditApplication app) {
			this.app = app;
			this.cachePath = RssBanditApplication.GetFeedFileCachePath();
		}

		#region public methods

		public static UserIdentity AnonymousIdentity {
			get {
				if (anonymous != null)
					return anonymous;

				anonymous = new UserIdentity();
				anonymous.Name = anonymous.RealName = "anonymous";
				anonymous.MailAddress = anonymous.ResponseAddress = String.Empty;
				anonymous.Organization = anonymous.ReferrerUrl = String.Empty;
				anonymous.Signature = String.Empty;

				return anonymous;
			}
		}

		public IDictionary CurrentIdentities {
			get { return this.app.FeedHandler.UserIdentity; }
		}
		public IDictionary CurrentNntpServers {
			get { return this.app.FeedHandler.NntpServers; }
		}

		/// <summary>
		/// Gets (filters) the current subscriptions for Nntp feeds of a specific nntp server.
		/// </summary>
		/// <param name="sd">NntpServerDefinition</param>
		/// <returns>List of feedsFeed objects, that match</returns>
		public IList CurrentSubscriptions(NntpServerDefinition sd) {
			//TODO: impl. CurrentSubscriptions()
			return new ArrayList();
		}

		/// <summary>
		/// Load the list of groups from either the cache, or if not available or forced 
		/// from the nntp server.
		/// </summary>
		/// <param name="sd">NntpServerDefinition</param>
		/// <param name="forceLoadFromServer">set to true, if a 'fresh' group list should be loaded from the server</param>
		/// <returns>List of groups a server offer</returns>
		public IList LoadNntpNewsGroups(NntpServerDefinition sd, bool forceLoadFromServer) {
			
			IList list = null;
			if (forceLoadFromServer) {
				list = FetchNewsGroupsFromServer(sd);
				if (list != null && list.Count > 0)
					SaveNewsGroupsToCache(list, sd);
				else
					RemoveCachedGroups(sd);
			} else {
				list = LoadNewsGroupsFromCache(sd);
			}

			return list;
		}

		public Uri BuildNntpRequestUri(NntpServerDefinition sd) {
			return BuildNntpRequestUri(sd, null);
		}
		public Uri BuildNntpRequestUri(NntpServerDefinition sd, string nntpGroup) {
			
			string schema = "nntp";
			if (sd.UseSSLSpecified && sd.UseSSL)
				schema = "nntps";
				
			int port = NewsHandler.NntpDefaultServerPort;
			if (sd.PortSpecified && sd.Port > 0 && sd.Port != NewsHandler.NntpDefaultServerPort)
				port = sd.Port;				
				
			UriBuilder uriBuilder = null;
			if (StringHelper.EmptyOrNull(nntpGroup))
				uriBuilder = new UriBuilder(schema, sd.Server, port);
			else
				uriBuilder = new UriBuilder(schema, sd.Server, port, nntpGroup);
			return uriBuilder.Uri;
		}
		#endregion

		#region private methods
		private string BuildCacheFileName(NntpServerDefinition sd) {
			//TODO
//			if (StringHelper.EmptyOrNull(sd.Server))
//				;
			return Path.Combine(cachePath, sd.Server);
		}

		private void RemoveCachedGroups(NntpServerDefinition sd) {
		
		}
		private void SaveNewsGroupsToCache(IList list, NntpServerDefinition sd) {
			//TODO
		}
		
		private IList LoadNewsGroupsFromCache(NntpServerDefinition sd) {
			//TODO
			return new ArrayList();
		}

		/// <summary>
		/// Really loads the group list from the server.
		/// </summary>
		/// <param name="sd">NntpServerDefinition</param>
		/// <returns>List of groups a server offer</returns>
		/// <exception cref="ArgumentNullException">If <see cref="sd">param sd</see> is null</exception>
		/// <exception cref="Exception">On any failure we get on request</exception>
		IList FetchNewsGroupsFromServer(NntpServerDefinition sd) {
			if (sd == null)
				throw new ArgumentNullException("sd");

			ArrayList result = new ArrayList(500);

			try {
				NntpWebRequest request = (NntpWebRequest) WebRequest.Create(BuildNntpRequestUri(sd)); 
				request.Method = "LIST"; 
					
				if (!StringHelper.EmptyOrNull(sd.AuthUser)) {
					string u = null, p = null;
					NewsHandler.GetNntpServerCredentials(sd, ref u, ref p);
					request.Credentials = NewsHandler.CreateCredentialsFrom(u, p);
				}

				//TODO: implement proxy support in NntpWebRequest
				//request.Proxy = app.Proxy;

				request.Timeout = 1000 * 60;	// default timeout: 1 minute
				if (sd.TimeoutSpecified)
					request.Timeout = sd.Timeout * 1000 * 60;	// sd.Timeout specified in minutes, but we need msecs

				WebResponse response = request.GetResponse();

				foreach(string s in NntpParser.GetNewsgroupList(response.GetResponseStream())){
					result.Add(s);
				}
			} catch (Exception ex) {
				_log.Error("Error receiving list of news groups from ", ex);
				throw;	// use for breakpoints
			}

			return result;
		}

		#endregion

		#region ShowDialog()'s 
		public void ShowIdentityDialog(IWin32Window owner) {
			this.ShowDialog(owner, NewsGroupSettingsView.Identity);
		}
		public void ShowNewsServerSubscriptionsDialog(IWin32Window owner) {
			this.ShowDialog(owner, NewsGroupSettingsView.NewsServerSubscriptions);
		}
		
		public void ShowDialog(IWin32Window owner, NewsGroupSettingsView view) {
			NewsGroupsConfiguration cfg = new NewsGroupsConfiguration(this, view);
			cfg.DefinitionsModified += new EventHandler(OnCfgDefinitionsModified);
			try {
				if (DialogResult.OK == cfg.ShowDialog(owner)) {
					if (!app.FeedlistModified)
						app.FeedlistModified = true;
				}
			} catch (Exception ex) {
				Trace.WriteLine("Exception in NewsGroupsConfiguration dialog: "+ex.Message);
			}
		}
		#endregion

		#region event handling
		private void OnCfgDefinitionsModified(object sender, EventArgs e) {
			
			NewsGroupsConfiguration cfg = sender as NewsGroupsConfiguration;

			// take over the copies from local userIdentities and nntpServers 
			// to app.FeedHandler.Identity and app.FeedHandler.NntpServers
			lock (app.FeedHandler.UserIdentity.SyncRoot) {
				app.FeedHandler.UserIdentity.Clear();
				if (cfg.ConfiguredIdentities != null) {
					foreach (UserIdentity ui in cfg.ConfiguredIdentities.Values) {
						app.FeedHandler.UserIdentity.Add(ui.Name, (UserIdentity)ui.Clone());
					}
				}
			}

			lock (app.FeedHandler.NntpServers.SyncRoot) {
				app.FeedHandler.NntpServers.Clear();
				if (cfg.ConfiguredNntpServers != null) {
					foreach (NntpServerDefinition sd in cfg.ConfiguredNntpServers.Values) {
						app.FeedHandler.NntpServers.Add(sd.Name, (NntpServerDefinition)sd.Clone());
					}	
				}
			}

			if (!app.FeedlistModified)
				app.FeedlistModified = true;
		}
		#endregion
	}
	#endregion
}
