#region CVS Version Header
/*
 * $Id: RssBanditPreferences.cs,v 1.17 2005/02/15 09:11:32 t_rendelmann Exp $
 * Last modified by $Author: t_rendelmann $
 * Last modified at $Date: 2005/02/15 09:11:32 $
 * $Revision: 1.17 $
 */
#endregion

using System;
using System.Drawing;
using System.Runtime.Serialization;
using System.Security.Cryptography;
using System.Security.Permissions;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Collections.Specialized;

using Logger = RssBandit.Common.Logging;

namespace RssBandit {
	
	/// <summary>
	/// What action will be taken to minimize the app to the system tray
	/// </summary>
	public enum HideToTray:int {
		None = 0,
		OnMinimize,
		OnClose
	}

	/// <summary>
	/// Auto-update interval settings
	/// </summary>
	public enum AutoUpdateMode:int {
		Manually = 0,
		OnApplicationStart,
		OnceIn14Days,
		OnceAMonth
	}

	/// <summary>
	/// Supported remote storage locations
	/// </summary>
	public enum RemoteStorageProtocolType:int {
		UNC = 0,
		FTP,
		dasBlog,	
		dasBlog_1_3,
		WebDAV
	}

	/// <summary>
	/// Embedded Web-Browser behavior on new window requests
	/// </summary>
	public enum BrowserBehaviorOnNewWindow:int {
		OpenNewTab = 0,
		OpenDefaultBrowser,
		OpenWithCustomExecutable
	}

	/// <summary>
	/// Options to control the display of toast notify windows
	/// </summary>
	public enum DisplayFeedAlertWindow:int {
		None = 0,
		AsConfiguredPerFeed,
		AsConfiguredPerCategory,
		All
	}

	/// <summary>
	/// RssBanditPreferences stores all the Bandit specific user preferences.
	/// </summary>
	[Serializable]
	public class RssBanditPreferences: ISerializable {
		
		private static readonly log4net.ILog _log = Logger.Log.GetLogger(typeof(RssBanditPreferences));

		// new: 1.3.x
		private string userIdentityForComments = String.Empty;

		// old: 1.2.x; see RssBanditApplication.CheckAndMigrateSettingsAndPreferences() 
		private string referer = String.Empty;
		private string userName = String.Empty;
		private string userMailAddress = String.Empty;

		private bool useProxy = false;
		private bool useIEProxySettings = false;
		private bool bypassProxyOnLocal = true;
		private string[] proxyBypassList = new string[]{};
		private bool proxyCustomCredentials = false;
		private string proxyAddress = String.Empty;
		private int proxyPort = 0;
		private string proxyUser = String.Empty;
		private string proxyPassword = String.Empty;

		private bool useRemoteStorage = false;
		private string remoteStorageUserName = String.Empty;
		private string remoteStoragePassword = String.Empty;
		private RemoteStorageProtocolType remoteStorageProtocol = RemoteStorageProtocolType.UNC;
		private string remoteStorageLocation = String.Empty;

		private string newsItemStylesheetFile = String.Empty;
		private bool reuseFirstBrowserTab = true;
		private bool newsItemOpenLinkInDetailWindow = true;
		private bool markItemsReadOnExit = false;
		private HideToTray hideToTrayAction = HideToTray.OnMinimize;
		// autoupdate properties
		private AutoUpdateMode autoUpdateFrequency = AutoUpdateMode.OnceIn14Days;
		private DateTime lastAutoUpdateCheck = DateTime.MinValue;

		private Font normalFont;
		private Font highlightFont;
		private Font flagFont;
		private Font errorFont;
		private Font refererFont;
		private Color normalFontColor = SystemColors.ControlText;
		private Color highlightFontColor = SystemColors.ControlText;
		private Color flagFontColor = Color.Green;
		private Color errorFontColor = Color.Red;
		private Color refererFontColor = Color.Blue;

		// general max item age: 30 days:
		private TimeSpan maxItemAge = TimeSpan.FromDays(30);	

		private BrowserBehaviorOnNewWindow browserBehaviorOnNewWindow = BrowserBehaviorOnNewWindow.OpenNewTab;
		private string browserCustomExecOnNewWindow = String.Empty;

		private bool feedRefreshOnStartup = false;

		bool allowJavascriptInBrowser, allowJavaInBrowser, allowActiveXInBrowser,
			allowBGSoundInBrowser, allowVideoInBrowser, allowImagesInBrowser = true;

		//private bool showConfiguredAlertWindows = true;	// old. Now a enum:DisplayFeedAlertWindow
		private bool showNewItemsReceivedBalloon = true;

		private DisplayFeedAlertWindow feedAlertWindow = DisplayFeedAlertWindow.AsConfiguredPerFeed;

		#region public properties

		public string UserIdentityForComments {
			get { return userIdentityForComments; }
			set { userIdentityForComments = value; }
		}

		public string Referer {
			get {	return referer;		}
			set {	referer = value;	}
		}

		public string UserName {
			get {	return userName;	}
			set {	userName = value;	}
		}

		public string UserMailAddress {
			get {	return userMailAddress;		}
			set {	userMailAddress = value;	}
		}

		public bool FeedRefreshOnStartup {			
			get {	return feedRefreshOnStartup; }
			set {	feedRefreshOnStartup = value;		}
		}

		public bool UseProxy {
			get {	return useProxy;		}
			set {	useProxy = value;		}
		}

		public bool UseIEProxySettings {
			get { return useIEProxySettings;	}
			set { useIEProxySettings = value;	}
		}

		public bool BypassProxyOnLocal {
			get {	return bypassProxyOnLocal;		}
			set {	bypassProxyOnLocal = value;		}
		}

		public string[] ProxyBypassList 
		{
			get {	return proxyBypassList;			}
			set {	proxyBypassList = value;		}
		}

		public bool ProxyCustomCredentials {
			get {	return proxyCustomCredentials;		}
			set {	proxyCustomCredentials = value;		}
		}

		public string ProxyAddress {
			get {	return proxyAddress;	}
			set {	proxyAddress = value;	}
		}

		public int ProxyPort {
			get {	return proxyPort;		}
			set {	proxyPort = value;	}
		}

		public string ProxyUser {
			get {	return proxyUser;		}
			set {	proxyUser = value;	}
		}

		public string ProxyPassword {
			get {	return proxyPassword;		}
			set {	proxyPassword = value;	}
		}
		
		public string NewsItemStylesheetFile {
			get {	return newsItemStylesheetFile;		}
			set {	newsItemStylesheetFile = value;	}
		}

		public bool ReuseFirstBrowserTab {
			get {	return reuseFirstBrowserTab;		}
			set {	reuseFirstBrowserTab = value;	}
		}	

		public bool MarkItemsReadOnExit {
			get { return markItemsReadOnExit;	}
			set { markItemsReadOnExit = value;	}
		}

		public bool NewsItemOpenLinkInDetailWindow {
			get { return newsItemOpenLinkInDetailWindow;	}
			set { newsItemOpenLinkInDetailWindow = value;	}
		}

		public HideToTray HideToTrayAction {
			get {	return hideToTrayAction;		}
			set {	hideToTrayAction = value;		}
		}

		public AutoUpdateMode AutoUpdateFrequency {
			get {	return autoUpdateFrequency;		}
			set {	autoUpdateFrequency = value;	}
		}

		public DateTime LastAutoUpdateCheck {
			get {	return lastAutoUpdateCheck;		}
			set {	lastAutoUpdateCheck = value;	}
		} 

		public Font NormalFont {
			get {	return normalFont;		}
			set {	normalFont = value;		}
		}

		public Font HighlightFont {
			get {	return highlightFont;		}
			set {	highlightFont = value;	}
		}

		public Font FlagFont {
			get {	return flagFont;		}
			set {	flagFont = value;		}
		}

		public Font RefererFont {
			get {	return refererFont;		}
			set {	refererFont = value;		}
		}

		public Font ErrorFont {
			get {	return errorFont;		}
			set {	errorFont = value;		}
		}

		public Color NormalFontColor {
			get {	return normalFontColor;		}
			set {	normalFontColor = value;	}
		}

		public Color HighlightFontColor {
			get {	return highlightFontColor;		}
			set {	highlightFontColor = value;		}
		}

		public Color FlagFontColor {
			get {	return flagFontColor;		}
			set {	flagFontColor = value;	}
		}

		public Color ErrorFontColor {
			get {	return errorFontColor;		}
			set {	errorFontColor = value;	}
		}

		public Color RefererFontColor {
			get {	return refererFontColor;		}
			set {	refererFontColor = value;	}
		}
		
		public TimeSpan MaxItemAge {
			get {	return maxItemAge;	}
			set {	maxItemAge = value;	}
		}

		public bool UseRemoteStorage {
			get { return useRemoteStorage; }
			set { useRemoteStorage = value; }
		}

		public string RemoteStorageUserName {
			get { return remoteStorageUserName; }
			set { remoteStorageUserName = value; }
		}

		public string RemoteStoragePassword {
			get { return remoteStoragePassword; }
			set { remoteStoragePassword = value; }
		}

		public RemoteStorageProtocolType RemoteStorageProtocol {
			get { return remoteStorageProtocol; }
			set { remoteStorageProtocol = value; }
		}

		public string RemoteStorageLocation {
			get { return remoteStorageLocation; }
			set { remoteStorageLocation = value; }
		}

		public BrowserBehaviorOnNewWindow BrowserOnNewWindow {
			get { return browserBehaviorOnNewWindow; }
			set { browserBehaviorOnNewWindow = value; }
		}

		public string BrowserCustomExecOnNewWindow  {
			get { return browserCustomExecOnNewWindow; }
			set { 
				if (value == null) 
					browserCustomExecOnNewWindow = String.Empty;
				 else	  
					browserCustomExecOnNewWindow = value; 
			}
		}

		public bool BrowserJavascriptAllowed { 
			get { return allowJavascriptInBrowser; }
			set {  allowJavascriptInBrowser = value; }
		}
		public bool BrowserJavaAllowed { 
			get { return allowJavaInBrowser; }
			set {  allowJavaInBrowser = value; }
		}
		public bool BrowserActiveXAllowed { 
			get { return allowActiveXInBrowser; }
			set {  allowActiveXInBrowser = value; }
		}
		public bool BrowserBGSoundAllowed { 
			get { return allowBGSoundInBrowser; }
			set {  allowBGSoundInBrowser = value; }
		}
		public bool BrowserVideoAllowed { 
			get { return allowVideoInBrowser; }
			set {  allowVideoInBrowser = value; }
		}
		public bool BrowserImagesAllowed { 
			get { return allowImagesInBrowser; }
			set {  allowImagesInBrowser = value; }
		}

		public DisplayFeedAlertWindow ShowAlertWindow { 
			get { return feedAlertWindow; }
			set {  feedAlertWindow = value; }
		}
		public bool ShowNewItemsReceivedBalloon { 
			get { return showNewItemsReceivedBalloon; }
			set {  showNewItemsReceivedBalloon = value; }
		}
		#endregion

		public RssBanditPreferences()	{
			normalFont = new Font(FontFamily.GenericSansSerif,9.75f ,FontStyle.Regular);
			highlightFont = new Font(normalFont,FontStyle.Bold);
			flagFont = new Font(normalFont,FontStyle.Regular);
			errorFont = new Font(normalFont,FontStyle.Regular);
			refererFont = new Font(normalFont,FontStyle.Regular);
		}

		protected RssBanditPreferences(SerializationInfo info, StreamingContext context) {
			int version = info.GetInt32("_PrefsVersion");

			UseProxy = info.GetBoolean("UseProxy");
			ProxyAddress = info.GetString("ProxyAddress");
			ProxyPort = info.GetInt32("ProxyPort");
			if (version >= 3) {	// uses encryption
				ProxyUser = EncryptionHelper.Decrypt(info.GetString("ProxyUser"));
				ProxyPassword = EncryptionHelper.Decrypt(info.GetString("ProxyPassword"));
			}
			else {
				// old: clear text
				ProxyUser = info.GetString("ProxyUser");
				ProxyPassword = info.GetString("ProxyPassword");
			}
			BypassProxyOnLocal = info.GetBoolean("BypassProxyOnLocal");
			ProxyCustomCredentials = info.GetBoolean("ProxyCustomCredentials");
			NewsItemStylesheetFile = info.GetString("NewsItemStylesheetFile");

			// see also version >= 16 below...
			// we still read them to enable migration
			UserName = info.GetString("UserName");
			UserMailAddress = info.GetString("UserMailAddress");
			Referer = info.GetString("Referer");

			if (version >= 2) {
				HideToTrayAction = (HideToTray)info.GetValue("HideToTrayAction",typeof(HideToTray));
			}
			if (version >= 4) {
				AutoUpdateFrequency = (AutoUpdateMode)info.GetValue("AutoUpdateFrequency",typeof(AutoUpdateMode));
				LastAutoUpdateCheck = (DateTime)info.GetDateTime("LastAutoUpdateCheck");
			}
			if (version >= 5) {
				NormalFont = (Font)info.GetValue("NormalFont",typeof(Font));
				NormalFontColor = (Color)info.GetValue("NormalFontColor",typeof(Color));
				HighlightFont = (Font)info.GetValue("HighlightFont",typeof(Font));
				HighlightFontColor = (Color)info.GetValue("HighlightFontColor",typeof(Color));
				FlagFont = (Font)info.GetValue("FlagFont",typeof(Font));
				FlagFontColor = (Color)info.GetValue("FlagFontColor",typeof(Color));
				// see also version >= 16 below...
			}
			if (version >= 6) {
				MaxItemAge = TimeSpan.FromTicks((long)info.GetInt64("MaxItemAge"));
			}
			if (version >= 7) { 
				UseRemoteStorage = info.GetBoolean("UseRemoteStorage");
				RemoteStorageUserName = info.GetString("RemoteStorageUserName");
				RemoteStoragePassword = info.GetString("RemoteStoragePassword");
				RemoteStorageProtocol = (RemoteStorageProtocolType)info.GetValue("RemoteStorageProtocol", RemoteStorageProtocol.GetType());
				RemoteStorageLocation = info.GetString("RemoteStorageLocation");
				// small workaround because of the enum index change in build 1.2.0.65 to support multiple dasBlog versions (moved to the end of the enum):
				if (UseRemoteStorage) {
					if (RemoteStorageProtocol == RemoteStorageProtocolType.FTP && !RemoteStorageLocation.StartsWith("ftp:"))
						RemoteStorageProtocol =  RemoteStorageProtocolType.dasBlog_1_3;	// old index mapped to dasBlog enum, before we added the dasBlog (1.5) support
				}

			}
			if (version >= 8) {
				BrowserOnNewWindow = (BrowserBehaviorOnNewWindow)info.GetValue("BrowserOnNewWindow", typeof(BrowserBehaviorOnNewWindow));
				BrowserCustomExecOnNewWindow = info.GetString("BrowserCustomExecOnNewWindow");
			}
			if (version >= 9) {
				NewsItemOpenLinkInDetailWindow = info.GetBoolean("NewsItemOpenLinkInDetailWindow");
			}
			if (version >= 10) {
				UseIEProxySettings = info.GetBoolean("UseIEProxySettings");
				FeedRefreshOnStartup = info.GetBoolean("FeedRefreshOnStartup");
			}
			if (version >= 11) {
				BrowserJavascriptAllowed = info.GetBoolean("BrowserJavascriptAllowed");
				BrowserJavaAllowed = info.GetBoolean("BrowserJavaAllowed");
				BrowserActiveXAllowed = info.GetBoolean("BrowserActiveXAllowed");
				BrowserBGSoundAllowed = info.GetBoolean("BrowserBGSoundAllowed");
				BrowserVideoAllowed = info.GetBoolean("BrowserVideoAllowed");
				BrowserImagesAllowed = info.GetBoolean("BrowserImagesAllowed");
			}
			if (version == 12) {
				bool showConfiguredAlertWindows = info.GetBoolean("ShowConfiguredAlertWindows");
				// migrate the old bool value to the new enum:
				if (showConfiguredAlertWindows) {
					ShowAlertWindow = DisplayFeedAlertWindow.AsConfiguredPerFeed;
				} else {
					ShowAlertWindow = DisplayFeedAlertWindow.None;
				}
			}
			if (version >= 12) {
				ShowNewItemsReceivedBalloon = info.GetBoolean("ShowNewItemsReceivedBalloon");
			}
			if (version >= 13) {
				ShowAlertWindow = (DisplayFeedAlertWindow)info.GetValue("ShowAlertWindow", typeof(DisplayFeedAlertWindow));
			}
			if (version >= 14){
				ProxyBypassList = (string[])info.GetValue("ProxyBypassList", typeof(string[]));
			}
			
			if (ProxyBypassList == null)
				ProxyBypassList = new string[]{};

			if(version >= 15){
				MarkItemsReadOnExit = info.GetBoolean("MarkItemsReadOnExit");
			}

			if (version >= 16) {
				UserIdentityForComments = info.GetString("UserIdentityForComments");
				ErrorFont = (Font)info.GetValue("ErrorFont",typeof(Font));
				ErrorFontColor = (Color)info.GetValue("ErrorFontColor",typeof(Color));
				RefererFont = (Font)info.GetValue("RefererFont",typeof(Font));
				RefererFontColor = (Color)info.GetValue("RefererFontColor",typeof(Color));
			} else {
				ErrorFont = new Font(this.NormalFont, FontStyle.Regular);
				RefererFont = new Font(this.NormalFont, FontStyle.Regular);
			}

			if (version >= 17) {
				ReuseFirstBrowserTab = info.GetBoolean("ReuseFirstBrowserTab");
			}
		}

		[SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter=true),
		 SecurityPermissionAttribute(SecurityAction.LinkDemand)]
		public virtual void GetObjectData(SerializationInfo info, StreamingContext context)	{
		
			info.AddValue("_PrefsVersion", 17);	// additional fontstyles/colors; default identity for feed comments, ReuseFirstBrowserTab
			info.AddValue("UseProxy", UseProxy);
			info.AddValue("ProxyAddress", ProxyAddress);
			info.AddValue("ProxyPort", ProxyPort);
			// v3 and higher use Encryption
			info.AddValue("ProxyUser", EncryptionHelper.Encrypt(ProxyUser));
			info.AddValue("ProxyPassword", EncryptionHelper.Encrypt(ProxyPassword));
			info.AddValue("BypassProxyOnLocal", BypassProxyOnLocal);
			info.AddValue("ProxyCustomCredentials", ProxyCustomCredentials);
			info.AddValue("NewsItemStylesheetFile", NewsItemStylesheetFile);
			info.AddValue("Referer", Referer);
			info.AddValue("UserName", UserName);
			info.AddValue("UserMailAddress", UserMailAddress);
			// v2 and higher
			info.AddValue("HideToTrayAction", HideToTrayAction);
			// v4 and higher use autoupdate
			info.AddValue("AutoUpdateFrequency", AutoUpdateFrequency);
			info.AddValue("LastAutoUpdateCheck", LastAutoUpdateCheck);
			// v5 and higher use font/colors
			info.AddValue("NormalFont", NormalFont);
			info.AddValue("NormalFontColor", NormalFontColor);
			info.AddValue("HighlightFont", HighlightFont);
			info.AddValue("HighlightFontColor", HighlightFontColor);
			info.AddValue("FlagFont", FlagFont);
			info.AddValue("FlagFontColor", FlagFontColor);
			// v6 and higher use the general MaxItemAge (we save the ticks as long)
			info.AddValue("MaxItemAge", MaxItemAge.Ticks);
			// v7 remote storage setting
			info.AddValue("UseRemoteStorage", UseRemoteStorage);
			info.AddValue("RemoteStorageUserName", RemoteStorageUserName);
			info.AddValue("RemoteStoragePassword", RemoteStoragePassword);
			info.AddValue("RemoteStorageProtocol", RemoteStorageProtocol);
			info.AddValue("RemoteStorageLocation", RemoteStorageLocation);
			// v8 custom BrowserOnNewWindow settings
			info.AddValue("BrowserOnNewWindow", BrowserOnNewWindow);
			info.AddValue("BrowserCustomExecOnNewWindow", BrowserCustomExecOnNewWindow);
			// v9 custom NewsItemOpenLinkInDetailWindow setting
			info.AddValue("NewsItemOpenLinkInDetailWindow", NewsItemOpenLinkInDetailWindow);
			// v10 UseIEProxySettings, FeedRefreshOnStartup
			info.AddValue("UseIEProxySettings", UseIEProxySettings);
			info.AddValue("FeedRefreshOnStartup", FeedRefreshOnStartup);
			// v11 with BrowserSecuritySettings
			info.AddValue("BrowserJavascriptAllowed", BrowserJavascriptAllowed);
			info.AddValue("BrowserJavaAllowed", BrowserJavaAllowed);
			info.AddValue("BrowserActiveXAllowed", BrowserActiveXAllowed);
			info.AddValue("BrowserBGSoundAllowed", BrowserBGSoundAllowed);
			info.AddValue("BrowserVideoAllowed", BrowserVideoAllowed);
			info.AddValue("BrowserImagesAllowed", BrowserImagesAllowed);
			// v12 with silent options (Alerts, Balloon)
			//info.AddValue("ShowConfiguredAlertWindows", ShowConfiguredAlertWindows); now new: ShowAlertWindow with enum
			info.AddValue("ShowNewItemsReceivedBalloon", ShowNewItemsReceivedBalloon);
			// v13 with fine grain silent options on Alerts
			info.AddValue("ShowAlertWindow", ShowAlertWindow);
			// v14 
			info.AddValue("ProxyBypassList", ProxyBypassList);
			//v15
			info.AddValue("MarkItemsReadOnExit", MarkItemsReadOnExit); 
			//v16
			info.AddValue("UserIdentityForComments", UserIdentityForComments); 
			info.AddValue("ErrorFont", ErrorFont);
			info.AddValue("ErrorFontColor", ErrorFontColor);
			info.AddValue("RefererFont", RefererFont);
			info.AddValue("RefererFontColor", RefererFontColor);
			//v17
			info.AddValue("ReuseFirstBrowserTab", ReuseFirstBrowserTab); 
		}

		private class EncryptionHelper {
			private static TripleDESCryptoServiceProvider _des;

			private EncryptionHelper(){}

			static EncryptionHelper() {
				_des = new TripleDESCryptoServiceProvider();
				_des.Key = _calcHash();
				_des.Mode = CipherMode.ECB;
//				for (int i = 0; i < _des.Key.GetLength(0); i++)
//					Trace.Write(_des.Key[i].ToString("x"));
//
//				Trace.WriteLine(" Mode: "+((int)_des.Mode).ToString());
			}

			public static string Decrypt(string str) {
				byte[] base64;
				byte[] bytes;
				string ret;

				if (str == null)
					ret = null;
				else {
					if (str.Length == 0)
						ret = String.Empty;
					else {
						try {
							base64 = Convert.FromBase64String(str);
							bytes = _des.CreateDecryptor().TransformFinalBlock(base64, 0, (int) base64.GetLength(0));
							ret = Encoding.Unicode.GetString(bytes);
						}
						catch (Exception e) {
							_log.Debug("Exception in Decrypt", e);
							ret = String.Empty;
						}
					}
				}
				return ret;
			}

			public static string Encrypt(string str) {
				byte[] inBytes;
				byte[] bytes;
				string ret;

				if (str == null)
					ret = null;
				else {
					if (str.Length == 0)
						ret = String.Empty;
					else {
						try {
							inBytes = Encoding.Unicode.GetBytes(str);
							bytes = _des.CreateEncryptor().TransformFinalBlock(inBytes, 0, (int) inBytes.GetLength(0));
							ret = Convert.ToBase64String(bytes);
						}
						catch (Exception e) {
							_log.Debug("Exception in Encrypt", e);
							ret = String.Empty;
						}
					}
				}
				return ret;
			}

			private static byte[] _calcHash() {
				string salt = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
				byte[] b = Encoding.Unicode.GetBytes(salt);
				int bLen = b.GetLength(0);
				
				// just to make the key somewhat "invisible" in Anakrino, we use the random class.
				// the seed (a prime number) makes it repro
				Random r = new Random(1500450271);	
				// result array
				byte[] res = new Byte[500];
				int i = 0;
				
				for (i = 0; i < bLen && i < 500; i++)
					res[i] = (byte)(b[i] ^ r.Next(30, 127));
				
				// padding:
				while (i < 500) {
					res[i] = (byte)r.Next(30, 127);
					i++;
				}

				MD5CryptoServiceProvider csp = new MD5CryptoServiceProvider();
				return csp.ComputeHash(res);
			}



		}


		
		/// <summary>
		/// Helps to deserialize the old RSSBandit.XXXTypes classes/types.
		/// It maps the type to the new namespace and same class.
		/// </summary>
		internal class DeserializationTypeBinder: SerializationBinder {
			public override System.Type BindToType(string assemblyName, string typeName) {
				System.Type typeToDeserialize = null;

				// For each assemblyName/typeName that you wish to deserialize to
				// to a different type, set typeToDeserialize to the desired type
				
				//String assemVer1 = Assembly.GetExecutingAssembly().FullName;
				string typeVer1 = "RSSBandit.";	// old namespace

				if (/* assemblyName == assemVer1 && */ typeName.IndexOf(typeVer1) == 0 ) {
					// old namespace found
					typeName = typeName.Replace(typeVer1, "RssBandit.");
				}

				// To return the type, do this:
				typeToDeserialize = Type.GetType(String.Format("{0}, {1}", 
					typeName, assemblyName));

				return typeToDeserialize;
			}
		}
	}
}
