#region CVS Version Header
/*
 * $Id: ThreadWorker.cs,v 1.11 2005/01/23 21:32:00 carnage4life Exp $
 * Last modified by $Author: carnage4life $
 * Last modified at $Date: 2005/01/23 21:32:00 $
 * $Revision: 1.11 $
 */
#endregion

using System;
using System.Collections;
using System.Diagnostics;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;

using RssBandit;
using NewsComponents;
using NewsComponents.Feed;
using Logger = RssBandit.Common.Logging;

namespace RssBandit
{

	#region ThreadWorker class
	/// <summary>
	/// ThreadWorker can be used to start a task in the background and report
	/// progress to the UI.
	/// </summary>
	public class ThreadWorker {

		/// <summary>
		/// My tasks to work on in background
		/// </summary>
		public enum Task {
			LoadFeedlist,
			LoadSpecialFeeds,
			RefreshFeeds,
			TransformFeed,			
			LoadCommentFeed,
			TransformCategory
		}

		/// <summary>
		/// Local Method for the actual work.
		/// </summary>
		private void LocalRunProcess() {
			switch (m_task) {
				
					#region LoadFeedlist
				case Task.LoadFeedlist:
					if (OnTaskProgress(1, 0) )
						return;	// cancelled
					try {
						RssBanditApplication app = (RssBanditApplication)this.m_taskArguments[0];
						bool applyTasksOnAppStartup = (bool)this.m_taskArguments[1];
						app.LoadFeedList();
						OnTaskFinished(1, 1, null, null);
						if (applyTasksOnAppStartup) {
							app.BeginRefreshFeeds(app.Preferences.FeedRefreshOnStartup);
						}
					} catch (Exception ex) {
						OnTaskFinished(1, 0, ex, null);
					}
					break;
					#endregion

					#region LoadSpecialFeeds
				case Task.LoadSpecialFeeds:
					if (OnTaskProgress(1, 0) )
						return;	// cancelled
					try {
						RssBanditApplication app = (RssBanditApplication)this.m_taskArguments[0];
						app.InitializeFlaggedItems();
						OnTaskFinished(1, 1, null, null);
					} catch (Exception ex) {
						OnTaskFinished(1, 0, ex, null);
					}
					break;
					#endregion

					#region RefreshFeeds
				case Task.RefreshFeeds:
					if (OnTaskProgress(1, 0) )
						return;	// cancelled
					try {
						RssBanditApplication app = (RssBanditApplication)this.m_taskArguments[0];
						bool force = (bool)this.m_taskArguments[1];
						app.FeedHandler.RefreshFeeds(force);
						OnTaskFinished(1, 1, null, null);
					} catch (Exception ex) {
						OnTaskFinished(1, 0, ex, null);
					}
					break;
					#endregion

					#region LoadCommentFeed
				case Task.LoadCommentFeed:
					if (OnTaskProgress(1, 0) )
						return;	// cancelled
					try {
						RssBanditApplication app = (RssBanditApplication)this.m_taskArguments[0];
						NewsItem item = (NewsItem)this.m_taskArguments[1];
						feedsFeed cmtFeed = new feedsFeed();
						cmtFeed.link = item.CommentRssUrl;
						cmtFeed.title = item.Feed.title;
						object result = app.FeedHandler.GetItemsForFeed(cmtFeed);
						OnTaskFinished(1, 1, null, new object[]{result, item, this.m_taskArguments[2], this.m_taskArguments[3]});
					} catch (Exception ex) {
						OnTaskFinished(1, 0, ex, new object[]{null, this.m_taskArguments[1], this.m_taskArguments[2], this.m_taskArguments[3]});
					}
					break;
					#endregion

					#region TransformFeed
				case Task.TransformFeed:					
					try {
						RssBanditApplication app = (RssBanditApplication)this.m_taskArguments[0];
						FeedInfo feed = (FeedInfo)this.m_taskArguments[1];
						TreeNode feedNode = (TreeNode)this.m_taskArguments[2];
						string stylesheet = (string) this.m_taskArguments[3];
						IComparer newsItemSorter = (IComparer) this.m_taskArguments[4];
						feed.ItemsList.Sort(newsItemSorter); 
						string html = app.FormatFeed(stylesheet, feed); 
						OnTaskFinished( feedNode, html, null);
					} catch (Exception ex) {
						OnTaskFinished(null, ex.Message, ex);
					}
					break;
					#endregion

					#region TransformCategory
				case Task.TransformCategory:					
					try {
						RssBanditApplication app = (RssBanditApplication)this.m_taskArguments[0];
						FeedInfoList feeds = (FeedInfoList)this.m_taskArguments[1];
						TreeNode feedNode = (TreeNode)this.m_taskArguments[2];
						string stylesheet = (string) this.m_taskArguments[3];
						IComparer newsItemSorter = (IComparer) this.m_taskArguments[4];
						
						foreach(FeedInfo feed in feeds){
							feed.ItemsList.Sort(newsItemSorter); 
						}
						
						string html = app.FormatFeeds(stylesheet, feeds); 
						
						OnTaskFinished( feedNode, html, null);
					} catch (Exception ex) {
						OnTaskFinished(null, ex.Message, ex);
					}
					break;
					#endregion
				default:
					Debug.Assert(false, "Unknown ThreadWorker task: " + m_task.ToString());
					break;
			}
		}

		#region ivar's
		/// <summary>
		/// Usually a form or a winform control that implements "Invoke/BeginInvoke".
		/// Used a the synchronization object for callbacks.
		/// </summary>
		ContainerControl m_sender = null;

		/// <summary>
		/// Task to work on
		/// </summary>
		Task m_task = 0;

		/// <summary>
		/// Task arguments (optional)
		/// </summary>
		object[] m_taskArguments;

		/// <summary>
		/// The delegate method (callback) on the sender to call
		/// </summary>
		ShowProgressHandler m_showProgressDelegate = null;

		/// <summary>
		/// The delegate method (callback) on the sender to call
		/// </summary>
		TransformedFeedsHandler m_transformedFeedsDelegate = null;
		#endregion

		#region ctor's
		/// <summary>
		/// Constructor used by caller using ThreadPool
		/// </summary>
		public ThreadWorker() {
		}
		/// <summary>
		/// Constructor called by callee using ThreadPool OR ThreadStart
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="senderDelegate"></param>
		/// <param name="task"></param>
		/// <param name="taskArgs"></param>
		public ThreadWorker ( ContainerControl sender, ShowProgressHandler senderDelegate, Task task, params object[] taskArgs) {
			m_sender = sender;
			m_showProgressDelegate = senderDelegate;
			m_task = task;
			m_taskArguments = taskArgs;
		}

		/// <summary>
		/// Constructor called by callee using ThreadPool OR ThreadStart
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="senderDelegate"></param>
		/// <param name="task"></param>
		public ThreadWorker ( ContainerControl sender, ShowProgressHandler senderDelegate, Task task){
			m_sender = sender;
			m_showProgressDelegate = senderDelegate;
			m_task = task;
			m_taskArguments = new object[]{};
		}

		/// <summary>
		/// Constructor called by callee using ThreadPool OR ThreadStart
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="senderDelegate"></param>
		/// <param name="task"></param>
		/// <param name="taskArgs"></param>
		public ThreadWorker ( ContainerControl sender, TransformedFeedsHandler senderDelegate, Task task, params object[] taskArgs) {
			m_sender = sender;
			m_transformedFeedsDelegate = senderDelegate;
			m_task = task;
			m_taskArguments = taskArgs;
		}

		/// <summary>
		/// Constructor called by callee using ThreadPool OR ThreadStart
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="senderDelegate"></param>
		/// <param name="task"></param>
		public ThreadWorker ( ContainerControl sender, TransformedFeedsHandler senderDelegate, Task task){
			m_sender = sender;
			m_transformedFeedsDelegate = senderDelegate;
			m_task = task;
			m_taskArguments = new object[]{};
		}
		#endregion

		#region public methods

		/// <summary>
		/// Queue a task using ThreadPool.
		/// </summary>
		/// <param name="task">Task to start</param>
		/// <param name="sender">ContainerControl</param>
		/// <param name="callback">ShowProgressHandler</param>
		/// <param name="taskArgs">optional task parameters</param>
		/// <returns>True, if the task was successfully queued up
		/// to the ThreadPool, else false.</returns>
		public static bool QueueTask(Task task, ContainerControl sender, ShowProgressHandler callback, params object[] taskArgs) {
			ThreadWorker wc = new ThreadWorker(sender, callback, task, taskArgs);
			bool rc = ThreadPool.QueueUserWorkItem( new WaitCallback (wc.RunProcess));
			return rc ;
		}

		/// <summary>
		/// Queue a task using a freshly created Thread.
		/// </summary>
		/// <param name="task">Task to start</param>
		/// <param name="sender">ContainerControl</param>
		/// <param name="callback">ShowProgressHandler</param>
		/// <param name="taskArgs">optional task parameters</param>
		public static void StartTask(Task task, ContainerControl sender, ShowProgressHandler callback, params object[] taskArgs) {
			ThreadWorker wc = new ThreadWorker(sender, callback, task, taskArgs);
			Thread t = new Thread( new ThreadStart(wc.RunProcess));			
			t.Priority = ThreadPriority.AboveNormal;
			t.IsBackground = true; //make them a daemon - prevent thread callback issues
			t.Start();
		}

		/// <summary>
		/// Queue a task using a freshly created Thread.
		/// </summary>
		/// <param name="task">Task to start</param>
		/// <param name="sender">ContainerControl</param>
		/// <param name="callback">TransformedFeedsHandler</param>
		/// <param name="taskArgs">optional task parameters</param>
		public static void StartTask(Task task, ContainerControl sender, TransformedFeedsHandler callback, params object[] taskArgs) {
			ThreadWorker wc = new ThreadWorker(sender, callback, task, taskArgs);
			Thread t = new Thread( new ThreadStart(wc.RunProcess));
			t.IsBackground = true; //make them a daemon - prevent thread callback issues
			t.Start();
		}
		
		/// <summary>
		/// Method for ThreadPool QueueWorkerItem
		/// </summary>
		/// <param name="obj"></param>
		public void RunProcess ( object obj ) {
			Thread.CurrentThread.IsBackground = true; //make them a daemon
			if (obj != null) {
				object[] objArray = (object[]) obj;
				m_sender = (ContainerControl) objArray[0];
				m_showProgressDelegate = (ShowProgressHandler) objArray[1];
				m_task = (Task) objArray[2];
			}
			LocalRunProcess();
		}

		/// <summary>
		/// Method for ThreadStart delegate
		/// </summary>
		public void RunProcess() {
			Thread.CurrentThread.IsBackground = true; //make them a daemon
			LocalRunProcess();
		}
		#endregion

		#region private helpers
		private bool OnTaskProgress(int total, int current) {
			return this.OnTaskProgress(total, current, null, null);
		}
		private bool OnTaskProgress(int total, int current, Exception error) {
			return this.OnTaskProgress(total, current, error, null);
		}
		private bool OnTaskProgress(int total, int current, Exception error, object result) {
			ShowProgressArgs args = new ShowProgressArgs(total, current, error, false, result); 
			object sender = System.Threading.Thread.CurrentThread;
			if (m_sender.InvokeRequired) {
				m_sender.Invoke(m_showProgressDelegate,  new object[] { sender, args } );
			} else {
				m_showProgressDelegate(sender, args);
			}
			return args.Cancel;
		}

		private void OnTaskFinished(TreeNode feedNode, string result, Exception error) {
			TransformedTreeArgs args = new TransformedTreeArgs(feedNode, result, error); 
			object sender = System.Threading.Thread.CurrentThread;
			if (m_sender.InvokeRequired) {
				m_sender.Invoke(m_transformedFeedsDelegate,  new object[] { sender, args } );
			} else {
				m_transformedFeedsDelegate(sender, args);
			}
		}

		private void OnTaskFinished(int total, int current, Exception error, object result) {
			ShowProgressArgs args = new ShowProgressArgs(total, current, error, true, result); 
			object sender = System.Threading.Thread.CurrentThread;
			if (m_sender.InvokeRequired) {
				m_sender.Invoke(m_showProgressDelegate,  new object[] { sender, args } );
			} else {
				m_showProgressDelegate(sender, args);
			}
		}
		#endregion

	}
	#endregion
	

	#region TransformedTreeArgs helper class 

	
	/// <summary>
	/// TransformedTreeArgs class transport the ThreadWorker progress arguments
	/// to the callee. 
	/// </summary>
	public class TransformedTreeArgs : CancelEventArgs {
		public TreeNode FeedNode; 
		public string Html; 
		public Exception Exception;

		public TransformedTreeArgs(TreeNode feedNode, string html, Exception exception){
			this.FeedNode = feedNode; 
			this.Html = html;
			this.Exception = exception;
		}
	}


	/// <summary>
	/// The callback routine using the TransformedTreeArgs class
	/// </summary>
	public delegate void TransformedFeedsHandler(object sender, TransformedTreeArgs e);

	#endregion 

	#region ShowProgressArgs helper class

	/// <summary>
	/// ShowProgressArgs class transport the ThreadWorker progress arguments
	/// to the callee. 
	/// </summary>
	public class ShowProgressArgs : CancelEventArgs {
		public Exception Exception;
		public int MaxProgress;
		public int CurrentProgress;
		public bool Done;
		public object Result;

		public ShowProgressArgs(int total, int current):
			this(total, current, null, false, null) {}
		public ShowProgressArgs(int total, int current, Exception exception):
			this(total, current, exception, false, null) {}
		public ShowProgressArgs(int total, int current, Exception exception, bool done, object result):base(false) {
			this.Exception = exception;
			this.Done = done;
			this.MaxProgress = total;
			this.CurrentProgress = current;
			this.Result = result;
		}
	}

	/// <summary>
	/// The callback routine using the ShowProgressArgs class
	/// </summary>
	public delegate void ShowProgressHandler(object sender, ShowProgressArgs e);
	
	#endregion

}
